% script file illustrate_Krylov.m
% 3D illustrations

A=[3 2 1; 2 6 1.3; 1 1.3 5];
A = [1 1 0; 1 12 10; 0 10 11];

%A(3,2) = 9;

b = [1; -11.01; -9.9];

x_ex = A\b;

% construct the Krylov space
x0 = 1*[1; -1; 0]; % for initial guess, otherwise need to shift
r0 = b-A*x0;

e1 = r0; 
e2 = A*e1; 
e3 = A*e2;

e1 = e1/norm(e1);
e2 = e2/norm(e2);
e3 = e3/norm(e3);

figure(1);
clf;
hold on


Q=line([x0(1); e1(1)+x0(1)],[x0(2); e1(2)+x0(2)], [x0(3); e1(3)+x0(3)]);
set(Q,'color','r','linewidth',2)
Q=line([x0(1); e2(1)+x0(1)],[x0(2); e2(2)+x0(2)], [x0(3); e2(3)+x0(3)]);
set(Q,'color','b','linewidth',2)
Q=line([x0(1); e3(1)+x0(1)],[x0(2); e3(2)+x0(2)], [x0(3); e3(3)+x0(3)]);
set(Q,'color','g')

axis equal
grid on
view(3)

plot3(x0(1),x0(2),x0(3),'mo','markersize',10,'markerfacecolor','m')


% go through the CG iteration and visualise the iterates

r0=b-A*x0; % initial residual
p = r0;

x=x0;

clrs=[1 0 0; 0 0 1; 0 1 0];

X = zeros(3,3); % here store i-th iterate in the i-th column

for i=1:3
  z = A*p;
  nu = (r0'*r0)/(p'*z);
  
  x = x + nu*p;

  X(:,i) = x;
  
  plot3(x(1),x(2),x(3),'*','color',clrs(i,:),'markersize',8);    
  %plot3(x(1),x(2),x(3),'*','color','g','markersize',8);    
  
  r = r0 - nu*z;
  
  r2 = r'*r;
    
  mu = r2/(r0'*r0);
  
  % reinitialise
  p = r + mu*p;
  r0=r;
    
end
  
plot3(x_ex(1),x_ex(2),x_ex(3),'co','markerfacecolor','c');

xlabel('x');
ylabel('y');
zlabel('z');
axis([1 3 -2 1 -0.5 1.5]);
axis equal

% visualise the plane between 1-st two Krylow vectors
% equation of the plane:
% Z = ax+by+c
% plane matrix: P
x1 = X(:,1); x2 = X(:,2);
P = [x0(1) x0(2) 1; x1(1) x1(2) 1; x2(1) x2(2) 1]; 
rhsP = [x0(3); x1(3); x2(3)];

% coefficients:
Pcfs = P\rhsP;
% visualise the surface
xx = linspace(-5,5);
yy = linspace(-5,5);
[XX,YY] = ndgrid(xx,yy);
ZZ = Pcfs(1)*XX + Pcfs(2)*YY + Pcfs(3);
M=mesh(XX,YY,ZZ);
set(M,'edgecolor',[0.8 0.8 0.8]);
set(M,'facecolor','none');

view([-8.84136 -3.63707 0.331367])

